// 
// MANDEL.cpp :
//    Calcul de l'ensemble de Mandelbrot de manière naïve, sauvegarde des
//    images générée dans des fichier images .png
//
//
// Necessite :
//    - GNU scientific library (GSL) --> inutile ici un tableau est largement
//    suffisant …
//    - libgd --> pour l'écriture des images 
// 
// Olivier Prot & Dominique Vieugue - (2005)
// 

#include <iostream>
#include <string>
#include <sstream>
// use of GSL matrix
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_matrix_double.h>

// gd plot lib
#include <gd.h>

using namespace std;

// colormap functions
double hatf(double x)
{
	if (x <=-1)
		return 0.0;
	else if (x>=1)
		return 0.0;
	else if (x<=0)
		return x+1.0;
	else
		return -x+1;
}

void setRGB(gdImagePtr im, int i, int j, int *red, int *green, int *blue)
{
  	int color;
  	// back into color format
  	color = gdTrueColor(*red, *green, *blue);
  	// set new pixel value
  	gdImageSetPixel ( im,  i,  j,  color);
}

// transforme la matrice en image .png
// f est le nom du fichier à écrire
void SavePNGMAT(gsl_matrix *MAT,const string *f)
{
  int i,j, s1, s2, N=f->length();
  char ff[N];
  double M,g,m;
  int r1,g1,b1; 
  FILE *pngout;
  gdImagePtr im;

  f->copy(ff, N);
  ff[N]='\0';

  M=gsl_matrix_max(MAT);
  m=gsl_matrix_min(MAT);
  // get matrix size
  s1 = MAT->size1;
  s2 = MAT->size2;

  im = gdImageCreateTrueColor(s1, s2);

  cout << s1 << "\t " << s2 << "\n";

  for (j=0;j<s1;j++)
    {
      for (i=0;i<s2;i++)
	{
	  g=gsl_matrix_get(MAT,i,j);
	  g-=m;
	  g/=(M-m);
	  r1 =(int)  (256*( hatf( 3*(g) +hatf(3*(g-1.0)))));
	  g1 = (int) (256*hatf( 3*(g-0.33)));
	  b1 = (int) (256*hatf(3*(g-0.66)));
	  setRGB(im, j, i, &r1, &g1, &b1);
	} 
    }
    pngout = fopen(ff, "wb");
  
  gdImagePng(im, pngout);
  
  /* Close the files. */
  fclose(pngout);

  /* Destroy the image in memory. */
  gdImageDestroy(im);

}


/////////////////////////////////////////////////////////////////////
// Illustration de la surchage d'opérateur pour la classe complexe //
/////////////////////////////////////////////////////////////////////
class complexe
{
public:
  double re;
  double im;
  complexe()
  {
    re=0;
    im=0;
  }
  complexe operator+();
  complexe operator*();
  double abs(complexe);
};

complexe operator+(complexe a,complexe b)
{
  complexe c;
  c.re=a.re+b.re;
  c.im=a.im+b.im;
  return c;
}

complexe operator*(complexe a,complexe b)
{
  complexe c;
  c.re=a.re*b.re-a.im*b.im;
  c.im=a.re*b.im+a.im*b.re;
  return c;
}

double abs(complexe a)
{
  double c;
  c=a.re*a.re+a.im*a.im;
  return c;
}

//____________________________________________________________

int main()
{
  gsl_matrix *MAT; 
  complexe z,zc,C;
  int ii, NITER=300;
  long int Nb=600,rrcol,cc,ll,colcol;
  double rrgg,c,l,div,px,py, tmp;

  stringstream oss;     

  string st;
  
  // matrix allocation
  MAT = gsl_matrix_alloc(Nb,Nb);

  rrgg=1.5/0.9;
  rrcol=100;
  div=rrgg/Nb;
  c=-rrgg;
  cc=1;
  for (ii=0;ii<NITER; ii++)
    {
      // 0.95 : facteur d'échelle entre les différents zoom
      rrgg=rrgg*0.95;
      div=rrgg/Nb;
      cc=1;
      //px=0.35967165; 
      //py=0.5993404; 

      // Nécessite un "bon" point pour le zoom
      px= - 0.577032172959 ; 
      py= - 0.634780405741 ; 

      c=py-rrgg;

      while (c<(rrgg+py) && cc<(Nb+1)) 
	{
	  l=px-rrgg;
	  ll=1;
	  while (l<(rrgg+px) && ll<(Nb+1))
	    {
	      // intialisation du nombre complexe
	      zc.re=l;
	      zc.im=c;
	      // mise à zéro : init. suite zk
	      z.re=0;
	      z.im=0;
	      colcol=0;
	      while ((colcol!=rrcol)&&(abs(z)<16))
            	{
		  // iteration de zk+1=zk**2+c 
		  // essayer aussi avec d'autres puissances
		  // z*z*z par exemple
		  z=z*z; 
		  z=z+zc;
		  colcol++;
		}
	      if (colcol==rrcol)
		{
		  colcol=0;
		}
	      
	      // compute color --> colormap ?
	      tmp=double(colcol%64);
	      tmp /= 64;

	      // remplissage de la matrice
	      //cout << cc << "\t" << ll << "\t" << tmp << "\n";
	      gsl_matrix_set(MAT, cc-1, ll-1, tmp);

	      //fprintf(fptr,"%d ",x[cc,ll]);
	      ll++;
	      l=l+2*div;
	    }
	  cc++;
	  c=c+2*div;
	}

      oss.str("");
      oss.width(4);
      oss.fill('0');
      oss << ii;
      st="fig"+oss.str()+".png";
      cout << st << "\t";
      // plot fig
      SavePNGMAT(MAT, &st);
    }
  // free memory 
  gsl_matrix_free(MAT);
  
  printf("\nFin des calculs \n");
  return 0;
}

